/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.IO;
using System.Reflection;
using NAnt.Core;
using NAnt.Core.Types;
using NAnt.Core.Attributes;
using TopCoder.Test.Framework.Servers;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// Defines a customized NAnt element. The element represents a server to be set up/cleaned up during the tests. The
    /// element must be nested under the "serverapptest" task.
    /// </summary>
    /// <remarks>
    /// The element contains attribute "class", "startserver", and "initserver". "class" is a mandatory attribute, which
    /// defines the class name of the <c>Server</c> implementation used to set up/clean up the test environment.
    /// "startserver" and "initserver" are optional boolean attributes. They define whether the server should be
    /// started/stopped and initialized/deinitialized during the tests respectively. If they are missing, by default the
    /// server should be started/stopped and initialized/deinitialized. The element may also contain "options",
    /// "credentials", and "sourcefiles" child elements. They are optional. Element "options" defines a name-value map,
    /// which contains the "option" elements defining names and values for concrete sub-classes. Element "credentials"
    /// defines a password-based authentication information, including a username, a password and a domain. It can be
    /// used by concrete sub-classes for authentication purpose. Element "sourcefiles" defines a set of files to be used
    /// by concrete sub-classes for various purposes. For example:<br/>
    /// <![CDATA[<server class="serverclass" startserver="no" initserver="yes">
    ///     <options>
    ///         <option name="name1" value="value1"/>
    ///         <option name="name2" value="value2"/>
    ///     </options>
    ///     <credentials username="user" password="pass"/>
    ///     <sourcefiles>
    ///         <!-- some files -->
    ///     </sourcefiles>
    /// </server>]]><br/>
    /// This class is not thread-safe.
    /// </remarks>
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [ElementName("server")]
    public class ServerElement : Element
    {
        /// <summary>
        /// Represents the class name of the <c>Server</c> implementation used to set up/clean up the test environment.
        /// </summary>
        private string className = null;

        /// <summary>
        /// Represents the <c>Server</c> implementation used to set up/clean up the test environment.
        /// </summary>
        /// <remarks>
        /// It is lazily initialized when it is needed.
        /// </remarks>
        private Server server = null;

        /// <summary>
        /// Represents a flag indicating whether <c>StartUp</c> and <c>ShutDown</c> method should be called before and
        /// after the tests.
        /// </summary>
        /// <remarks>
        /// When it is <c>true</c>, the two methods should be called.
        /// </remarks>
        private bool startServer = true;

        /// <summary>
        /// Represents a flag indicating whether <c>InitData</c> and <c>CleanUpData</c> method should be called before
        /// and after the tests.
        /// </summary>
        /// <remarks>
        /// When it is <c>true</c>, the two methods should be called.
        /// </remarks>
        private bool initServer = true;

        /// <summary>
        /// Represents the collection of all option names and values.
        /// </summary>
        private readonly OptionCollection options = new OptionCollection();

        /// <summary>
        /// Represents the credential information used to authenticate the server.
        /// </summary>
        private Credential credentials = new Credential();

        /// <summary>
        /// Represents the set of files configured for the server.
        /// </summary>
        private FileSet sourceFiles = new FileSet();

        /// <summary>
        /// Gets or sets the class name of the <c>Server</c> implementation used to set up/clean up the test
        /// environment.
        /// </summary>
        /// <remarks>
        /// The class name is extracted from the attribute "class". It is mandatory.
        /// The setter should only be used by NAnt core. There is no validation, since NAnt core will guarantee that
        /// the values are valid.
        /// </remarks>
        /// <value>The class name of the <c>Server</c> implementation used to set up/clean up the test environment.
        /// </value>
        [TaskAttribute("class", Required = true)]
        public string ClassName
        {
            get
            {
                return className;
            }
            set
            {
                className = value;
            }
        }

        /// <summary>
        /// Gets or sets the flag indicating whether <c>StartUp</c> and <c>ShutDown</c> method should be called before
        /// and after the tests.
        /// </summary>
        /// <remarks>
        /// The flag is extracted from the attribute "startserver".
        /// The setter should only be used by NAnt core. There is no validation, since NAnt core will guarantee that
        /// the values are valid.
        /// </remarks>
        /// <value><c>true</c> if <c>StartUp</c> and <c>ShutDown</c> should be called; <c>false</c> otherwise.</value>
        [TaskAttribute("startserver")]
        public bool StartServer
        {
            get
            {
                return startServer;
            }
            set
            {
                startServer = value;
            }
        }

        /// <summary>
        /// Gets or sets the flag indicating whether <c>InitData</c> and <c>CleanUpData</c> method should be called
        /// before and after the tests.
        /// </summary>
        /// <remarks>
        /// The flag is extracted from the attribute "initserver".
        /// The setter should only be used by NAnt core. There is no validation, since NAnt core will guarantee that
        /// the values are valid.
        /// </remarks>
        /// <value><c>true</c> if <c>InitData</c> and <c>CleanUpData</c> should be called; <c>false</c> otherwise.
        /// </value>
        [TaskAttribute("initserver")]
        public bool InitServer
        {
            get
            {
                return initServer;
            }
            set
            {
                initServer = value;
            }
        }

        /// <summary>
        /// Gets the collection of all option names and values.
        /// </summary>
        /// <remarks>
        /// The option names and values are extracted from "option" elements nested under "options" element.
        /// </remarks>
        /// <value>The collection of all option names and values.</value>
        [BuildElementCollection("options", "option")]
        public OptionCollection Options
        {
            get
            {
                return options;
            }
        }

        /// <summary>
        /// Gets or sets the credential information used to authenticate the server.
        /// </summary>
        /// <remarks>
        /// The credential information is extracted from "credentials" element.
        /// The setter should only be used by NAnt core. There is no validation, since NAnt core will guarantee that
        /// the values are valid.
        /// </remarks>
        /// <value>The credential information used to authenticate the server.</value>
        [BuildElement("credentials")]
        public Credential Credentials
        {
            get
            {
                return credentials;
            }
            set
            {
                credentials = value;
            }
        }

        /// <summary>
        /// Gets or sets a set of files configured for the server.
        /// </summary>
        /// <remarks>
        /// The set of files is extracted from "sourcefiles" element. Its usage depends on the concrete implementation.
        /// The setter should only be used by NAnt core. There is no validation, since NAnt core will guarantee that
        /// the values are valid.
        /// </remarks>
        /// <value>A set of files configured for the server.</value>
        [BuildElement("sourcefiles")]
        public FileSet SourceFiles
        {
            get
            {
                return sourceFiles;
            }
            set
            {
                sourceFiles = value;
            }
        }

        /// <summary>
        /// Creates a new instance of <c>ServerElement</c> class.
        /// </summary>
        public ServerElement()
        {
        }

        /// <summary>
        /// Sets the project of an NAnt element.
        /// </summary>
        /// <remarks>
        /// When the project of the NAnt element is not set, it is set to the project of this element.
        /// </remarks>
        /// <param name="element">The NAnt element whose project is set.</param>
        private void SetElementProject(Element element)
        {
            if (element.Project == null)
            {
                element.Project = Project;
            }
        }

        /// <summary>
        /// Gets an instance of the <c>Server</c> implementation. The class name of the instance is configured using the
        /// "class" attribute.
        /// </summary>
        /// <remarks>
        /// The server instance is created only once. After which, the same instance is returned every time. The server
        /// instance is configured according to the configuration in this element. The options, start/init server flag,
        /// the credentials, and the source files are set in the server instance. There is no checking on class name,
        /// since NAnt should guarantee that class name is specified.
        /// </remarks>
        /// <returns>A server instance configured by this element.</returns>
        public Server GetServerInstance()
        {
            if (server == null)
            {
                // When the server instance is not created, create it.
                try
                {
                    // Create the instance.
                    server = (Server)Activator.CreateInstance(Type.GetType(className, true));
                }
                catch (TypeLoadException e)
                {
                    throw new ServerException("The class of the server cannot be loaded.", e);
                }
                catch (TargetInvocationException e)
                {
                    throw new ServerException("The class initializer or the constructor of the server causes an error.",
                        e);
                }
                catch (ArgumentException e)
                {
                    throw new ServerException("The class name of the server is invalid.", e);
                }
                catch (IOException e)
                {
                    throw new ServerException("The assembly of the type cannot be loaded.", e);
                }
                catch (MemberAccessException e)
                {
                    throw new ServerException("The server class cannot be instantiated via default constructor.", e);
                }
                catch (NotSupportedException e)
                {
                    throw new ServerException("The server class is a special class.", e);
                }
                catch (InvalidCastException e)
                {
                    throw new ServerException("The class is not a server class.", e);
                }

                // Set the project of the properties
                SetElementProject(credentials);
                SetElementProject(sourceFiles);

                // Set the properties
                server.Options = options;
                server.StartServer = startServer;
                server.InitServer = initServer;
                server.SourceFiles = sourceFiles;
                server.Credentials = credentials;
                server.Project = Project;
            }

            return server;
        }
    }
}
