/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// Defines an abstract database server by extending the <c>Server</c> class. It provides access to common options
    /// required by most database servers. The options include database driver, database connection string, SQL scripts
    /// used to initialize the data in the database, and the SQL scripts used to clean the data in the database. The
    /// option names are "db_driver", "db_connection_str", "sql_init_script", and "sql_cleanup_script" respectively.
    /// The values of these options are strings.
    /// </summary>
    /// <remarks>
    /// This class is not thread-safe.
    /// </remarks>
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public abstract class DatabaseServer : Server
    {
        /// <summary>
        /// Represents the option name where the database driver is stored.
        /// </summary>
        public const string DB_DRIVER = "db_driver";

        /// <summary>
        /// Represents the option name where the database connection string is stored.
        /// </summary>
        public const string DB_CONNECTION_STR = "db_connection_str";

        /// <summary>
        /// Represents the option name where the SQL script used to initialize the database is stored.
        /// </summary>
        public const string SQL_INIT_SCRIPT = "sql_init_script";

        /// <summary>
        /// Represents the option name where the SQL script used to clean the database is stored.
        /// </summary>
        public const string SQL_CLEANUP_SCRIPT = "sql_cleanup_script";

        /// <summary>
        /// Gets the database driver of this server.
        /// </summary>
        /// <remarks>
        /// Its usage depends on the concrete sub-class. It is extracted from the option "db_driver".
        /// </remarks>
        /// <value>The database driver of the server.</value>
        /// <exception cref="ServerException">The option "db_driver" is missing.</exception>
        public string DatabaseDriver
        {
            get
            {
                return TestingFrameworkHelper.GetOptionValue(Options, DB_DRIVER);
            }
        }

        /// <summary>
        /// Gets the database connection string of this server.
        /// </summary>
        /// <remarks>
        /// Its usage depends on the concrete sub-class. It is extracted from the option "db_connection_str".
        /// </remarks>
        /// <value>The database connection string of the server.</value>
        /// <exception cref="ServerException">The option "db_connection_str" is missing.</exception>
        public string DatabaseConnectionString
        {
            get
            {
                return TestingFrameworkHelper.GetOptionValue(Options, DB_CONNECTION_STR);
            }
        }

        /// <summary>
        /// Gets the SQL script used to initialize the database.
        /// </summary>
        /// <remarks>
        /// Its usage depends on the concrete sub-class. It is extracted from the option "sql_init_script".
        /// </remarks>
        /// <value>The SQL script used to initialize the database.</value>
        /// <exception cref="ServerException">The option "sql_init_script" is missing.</exception>
        public string SqlInitScript
        {
            get
            {
                return TestingFrameworkHelper.GetOptionValue(Options, SQL_INIT_SCRIPT);
            }
        }

        /// <summary>
        /// Gets the SQL script used to clean the database.
        /// </summary>
        /// <remarks>
        /// Its usage depends on the concrete sub-class. It is extracted from the option "sql_cleanup_script".
        /// </remarks>
        /// <value>The SQL script used to clean the database.</value>
        /// <exception cref="ServerException">The option "sql_cleanup_script" is missing.</exception>
        public string SqlCleanUpScript
        {
            get
            {
                return TestingFrameworkHelper.GetOptionValue(Options, SQL_CLEANUP_SCRIPT);
            }
        }

        /// <summary>
        /// Creates a new instance of <c>DatabaseServer</c> class.
        /// </summary>
        protected DatabaseServer()
        {
        }
    }
}

