/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using NAnt.Core;
using NAnt.Core.Types;

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// Defines an abstract server, which has the functionality to start/stop the server and initialize/clean up the
    /// data of the server. It contains all information in <c>ServerElement</c> class, except the class name. The
    /// information includes start/init flags, options, credentials, and a set of source files. It also provides access
    /// to the port and address of the server. They are defined in the options using name "port" and "address". The port
    /// must be a non-negative integer. Besides these, a time-out value is also defined by the name "timeout" in the
    /// options. It also must be a non-negative integer.
    /// <seealso cref="ServerElement"/>
    /// </summary>
    /// <remarks>
    /// This class is not thread-safe.
    /// </remarks>
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public abstract class Server
    {
        /// <summary>
        /// Represents a flag indicating whether <c>StartUp</c> and <c>ShutDown</c> method should be called before and
        /// after the tests.
        /// </summary>
        /// <remarks>
        /// When it is <c>true</c>, the two methods should be called.
        /// </remarks>
        private bool startServer = true;

        /// <summary>
        /// Represents a flag indicating whether <c>InitData</c> and <c>CleanUpData</c> method should be called before
        /// and after the tests.
        /// </summary>
        /// <remarks>
        /// When it is <c>true</c>, the two methods should be called.
        /// </remarks>
        private bool initServer = true;

        /// <summary>
        /// Represents the collection of all option names and values.
        /// </summary>
        private OptionCollection options = null;

        /// <summary>
        /// Represents the credential information used to authenticate the server.
        /// </summary>
        private Credential credentials = null;

        /// <summary>
        /// Represents the set of files configured for the server.
        /// </summary>
        private FileSet sourceFiles = null;

        /// <summary>
        /// Represents the NAnt project which the server belongs to.
        /// </summary>
        private Project project = null;

        /// <summary>
        /// Gets or sets the NAnt project which the server belongs to.
        /// </summary>
        /// <value>The NAnt project which the server belongs to.</value>
        public Project Project
        {
            get
            {
                return project;
            }
            set
            {
                project = value;
            }
        }

        /// <summary>
        /// Gets or sets the flag indicating whether <c>StartUp</c> and <c>ShutDown</c> method should be called before
        /// and after the tests.
        /// </summary>
        /// <value><c>true</c> if <c>StartUp</c> and <c>ShutDown</c> should be called; <c>false</c> otherwise.</value>
        public bool StartServer
        {
            get
            {
                return startServer;
            }
            set
            {
                startServer = value;
            }
        }

        /// <summary>
        /// Gets or sets the flag indicating whether <c>InitData</c> and <c>CleanUpData</c> method should be called
        /// before and after the tests.
        /// </summary>
        /// <value><c>true</c> if <c>InitData</c> and <c>CleanUpData</c> should be called; <c>false</c> otherwise.
        /// </value>
        public bool InitServer
        {
            get
            {
                return initServer;
            }
            set
            {
                initServer = value;
            }
        }

        /// <summary>
        /// Gets the collection of all option names and values.
        /// </summary>
        /// <remarks>
        /// Its usage depends on the concrete implementation. Since this class is initialized by the
        /// <c>ServerElement</c> class, this property is definitely not <c>null</c>.
        /// </remarks>
        /// <value>The collection of all option names and values.</value>
        public OptionCollection Options
        {
            get
            {
                return options;
            }
            set
            {
                options = value;
            }
        }

        /// <summary>
        /// Gets or sets the credential information used to authenticate the server.
        /// </summary>
        /// <remarks>
        /// Its usage depends on the concrete implementation.
        /// </remarks>
        /// <value>The credential information used to authenticate the server.</value>
        public Credential Credentials
        {
            get
            {
                return credentials;
            }
            set
            {
                credentials = value;
            }
        }

        /// <summary>
        /// Gets or sets a set of files configured for the server.
        /// </summary>
        /// <remarks>
        /// Its usage depends on the concrete implementation.
        /// </remarks>
        /// <value>A set of files configured for the server.</value>
        public FileSet SourceFiles
        {
            get
            {
                return sourceFiles;
            }
            set
            {
                sourceFiles = value;
            }
        }

        /// <summary>
        /// Represents the option name where the time out value is defined. The time out value must be a non-negative
        /// integer.
        /// </summary>
        public const string TIMEOUT = "timeout";

        /// <summary>
        /// Represents the option name where the port of the server is defined. The port must be a non-negative integer.
        /// </summary>
        public const string PORT = "port";

        /// <summary>
        /// Represents the option name where the address of the server is defined.
        /// </summary>
        public const string ADDRESS = "address";

        /// <summary>
        /// Represents the maximum allowed port number in TCP/IP.
        /// </summary>
        private const int MAX_PORT = 65535;

        /// <summary>
        /// Gets the time out value configured for this server.
        /// </summary>
        /// <remarks>
        /// Its usage depends on the concrete sub-class. It must be a non-negative integer. It is extracted from the
        /// option "timeout".
        /// </remarks>
        /// <value>The time out value configured for the server.</value>
        /// <exception cref="ServerException">The option "timeout" is missing; or the value is not a non-negative
        /// integer.</exception>
        public int Timeout
        {
            get
            {
                return TestingFrameworkHelper.GetOptionIntValue(Options, TIMEOUT, int.MaxValue);
            }
        }

        /// <summary>
        /// Gets the port of this server.
        /// </summary>
        /// <remarks>
        /// Its usage depends on the concrete sub-class. It must be a non-negative integer. It is extracted from the
        /// option "port".
        /// </remarks>
        /// <value>The port of the server.</value>
        /// <exception cref="ServerException">The option "port" is missing; or the value is not a non-negative
        /// integer; or the port number is out of the TCP/IP range.</exception>
        public int Port
        {
            get
            {
                return TestingFrameworkHelper.GetOptionIntValue(Options, PORT, MAX_PORT);
            }
        }

        /// <summary>
        /// Gets the address of this server.
        /// </summary>
        /// <remarks>
        /// Its usage depends on the concrete sub-class. It is extracted from the option "address".
        /// </remarks>
        /// <value>The address of the server.</value>
        /// <exception cref="ServerException">The option "address" is missing.</exception>
        public string Address
        {
            get
            {
                return TestingFrameworkHelper.GetOptionValue(Options, ADDRESS);
            }
        }

        /// <summary>
        /// Creates a new instance of <c>Server</c> class.
        /// </summary>
        protected Server()
        {
        }

        /// <summary>
        /// Starts up the server.
        /// </summary>
        /// <remarks>
        /// This method is called only if the start flag is set to <c>true</c>. It is called before the initialization
        /// of the server and the tests.
        /// </remarks>
        /// <exception cref="ServerException">Any error occurs when starting up the server.</exception>
        public abstract void StartUp();

        /// <summary>
        /// Shuts down the server.
        /// </summary>
        /// <remarks>
        /// This method is called only if the start flag is set to <c>true</c>. It is called after the deinitialization
        /// of the server and the tests.
        /// </remarks>
        /// <exception cref="ServerException">Any error occurs when shuting down the server.</exception>
        public abstract void ShutDown();

        /// <summary>
        /// Initializes the data of the server.
        /// </summary>
        /// <remarks>
        /// This method is called only if the init flag is set to <c>true</c>. It is called after starting the server
        /// and before the tests.
        /// </remarks>
        /// <exception cref="ServerException">Any error occurs when initializing the data.</exception>
        public abstract void InitData();

        /// <summary>
        /// Cleans up the data of the server.
        /// </summary>
        /// <remarks>
        /// This method is called only if the init flag is set to <c>true</c>. It is called before stopping the server
        /// and after the tests.
        /// </remarks>
        /// <exception cref="ServerException">Any error occurs when cleaning up the data.</exception>
        public abstract void CleanUpData();
    }
}
