/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.IO;
using System.ComponentModel;
using System.ServiceProcess;
using NAnt.Core;
using NAnt.Core.Tasks;
using NAnt.Core.Types;
using System.Collections;

namespace TopCoder.Test.Framework.Servers.Web
{
    /// <summary>
    /// Defines a class used to start/stop IIS server before and after the tests. Before starting the IIS server, the
    /// files in the source file set are copied into the deployment directory of the IIS server. The deployment
    /// directory is relative to the base directory of the NAnt project file. After stopping the IIS server, the
    /// deployed files are deleted from the deployment directory. The IIS server are controlled via the Windows service
    /// controller. The service name is defined by the option "service_name". If it is missing or empty string, the
    /// default "w3svc" service name is used. Starting/stopping the IIS server would wait the amount of time defined by
    /// "timeout" option. It is defined in milliseconds. When such option is missing or empty string, the default value
    /// 60000 (one minute) is used. When the init data flag is <c>true</c>, the deployment directory must be specified
    /// and the source files are copied/deleted. Otherwise, the source files are not copied/deleted.
    /// </summary>
    /// <remarks>
    /// This class is not thread-safe.
    /// </remarks>
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public class IISServer : ApplicationServer
    {
        /// <summary>
        /// Represents the option name where the service name of the IIS server is stored.
        /// </summary>
        public const string SERVICE_NAME = "service_name";

        /// <summary>
        /// Represents the default service name of the IIS server to be controlled.
        /// </summary>
        private const string DEFAULT_SERVICE_NAME = "w3svc";

        /// <summary>
        /// Represents the default time out of the IIS server to be started or stopped.
        /// </summary>
        private const int DEFAULT_TIMEOUT = 60000;

        /// <summary>
        /// Creates a new instance of <c>IISServer</c> class.
        /// </summary>
        public IISServer()
        {
        }

        /// <summary>
        /// Defines a customized comparer used to sort the string according to the length. The shorter string is
        /// considered as smaller.
        /// </summary>
        /// <remarks>
        /// This class is thread-safe, since it is stateless.
        /// </remarks>
        /// <author>TCSDEVELOPER</author>
        /// <version>1.0</version>
        /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
        private class StringLengthComparer : IComparer
        {
            /// <summary>
            /// Compares two objects according to their string representation. The shorter string representation is
            /// considered as smaller.
            /// </summary>
            /// <param name="a">The first object to be compared.</param>
            /// <param name="b">The second object to be compared.</param>
            /// <returns>A negative value if <c>a</c> is smaller than <c>b</c>; or a positive value if <c>a</c> is
            /// greater than <c>b</c>; or 0 if <c>a</c> equals <b>b</b>.</returns>
            public int Compare(object a, object b)
            {
                return a.ToString().Length - b.ToString().Length;
            }
        }

        /// <summary>
        /// Starts up the IIS server.
        /// </summary>
        /// <remarks>
        /// This method is called only if the start flag is set to <c>true</c>. It is called before the initialization
        /// of the server and the tests. Before starting the IIS server, the web site is deployed to the deployment
        /// directory. All files in the source file set are copied to the deployment directory. It waits the service to
        /// be started until the time out expires. If the time out value is not specified or is empty string, the
        /// default time out 60000 is used. The time out value is defined in milliseconds.
        /// </remarks>
        /// <exception cref="ServerException">Deploying the web site fails; or the IIS server is already started; or
        /// the IIS service is starting, stopping, resuming, or pausing; or the IIS server fails to be started after
        /// the configured time out; or the IIS server cannot be started.</exception>
        public override void StartUp()
        {
            // First, copy the deployment files to the deployment directory.
            if (InitServer)
            {
                // If there is source files to be copied, copy the files
                CopyTask copy = new CopyTask();

                copy.CopyFileSet = SourceFiles;
                copy.ToDirectory = DeploymentDirectory;
                copy.Overwrite = true;
                copy.Project = Project;

                try
                {
                    copy.Execute();
                }
                catch (BuildException e)
                {
                    throw new ServerException("Deploying the web site fails.", e);
                }
                catch (IOException e)
                {
                    // When fail to create an empty directory, it would cause I/O error.
                    throw new ServerException("Copying file causes I/O error.", e);
                }
            }

            // Then, start the IIS service and wait
            ServiceController controller;
            int timeOut;

            PrepareStartStop(out controller, out timeOut);

            try
            {
                switch (controller.Status)
                {
                    case ServiceControllerStatus.Paused:
                        // When the service is paused, it should be continued.
                        controller.Continue();

                        break;

                    case ServiceControllerStatus.Stopped:
                        // When the service is stopped, it should be started.
                        controller.Start();

                        break;

                    default:
                        // Otherwise, the service is either in a pending status, or running, which is illegal
                        throw new ServerException("The IIS service is already running or busy changing status.");
                }

                // Wait for the status change
                controller.WaitForStatus(ServiceControllerStatus.Running, TimeSpan.FromMilliseconds(timeOut));
            }
            catch (InvalidOperationException e)
            {
                throw new ServerException("The service status cannot be started/resumed.", e);
            }
            catch (Win32Exception e)
            {
                throw new ServerException("The service cannot be started/resumed due to Win32 error.", e);
            }
            catch (System.ServiceProcess.TimeoutException e)
            {
                throw new ServerException("The service is not started within the time limit.", e);
            }
            finally
            {
                // Free all handles and other resources.
                controller.Dispose();
            }
        }

        /// <summary>
        /// Shuts down the IIS server.
        /// </summary>
        /// <remarks>
        /// This method is called only if the start flag is set to <c>true</c>. It is called after the deinitialization
        /// of the server and the tests. After stopping the IIS server, the web site is deleted from the deployment
        /// directory. All files in the source file set are deleted from the deployment directory. It waits the service
        /// to be stopped until the time out expires. If the time out value is not specified or is empty string, the
        /// default time out 60000 is used. The time out value is defined in milliseconds.
        /// </remarks>
        /// <exception cref="ServerException">Deleting the web site fails; or the IIS server cannot be stopped; or the
        /// IIS server fails to be stopped after the configured time out.</exception>
        public override void ShutDown()
        {
            // First stop the server
            ServiceController controller;
            int timeOut;

            PrepareStartStop(out controller, out timeOut);

            try
            {
                if (!controller.CanStop)
                {
                    // When the service cannot be stopped.
                    throw new ServerException("The IIS server cannot be stopped.");
                }

                // Stop the service
                controller.Stop();

                // Wait for the status change
                controller.WaitForStatus(ServiceControllerStatus.Stopped, TimeSpan.FromMilliseconds(timeOut));
            }
            catch (InvalidOperationException e)
            {
                throw new ServerException("The IIS service cannot be stopped.", e);
            }
            catch (Win32Exception e)
            {
                throw new ServerException("The IIS service cannot be stopped due to Win32 error.", e);
            }
            catch (System.ServiceProcess.TimeoutException e)
            {
                throw new ServerException("The service is not stopped within the time limit.", e);
            }
            finally
            {
                // Free all handles and other resources.
                controller.Dispose();
            }

            // Then delete the deployed files
            if (InitServer)
            {
                // If there are files deployed, delete the files
                DeleteTask delete = new DeleteTask();
                FileSet fileSet = new FileSet();
                DirectoryInfo baseDir = new DirectoryInfo(SourceFiles.BaseDirectory);
                string deployBase = Project.GetFullPath(DeploymentDirectory);

                // Get all copied files in the original directory
                foreach (string file in SourceFiles.FileNames)
                {
                    fileSet.Includes.Add(GetPath(baseDir, file));
                }

                // Set the base directory to the deployed directory
                fileSet.BaseDirectory = deployBase;
                delete.DeleteFileSet = fileSet;
                delete.Project = Project;

                try
                {
                    // Delete the files in the deployed directory
                    delete.Execute();
                }
                catch (BuildException e)
                {
                    throw new ServerException("Deleting the deployed web site fails.", e);
                }

                // Remove all copied directory manually
                ArrayList dirs = new ArrayList();

                // Get all copied directories in the original directory
                foreach (string dir in SourceFiles.DirectoryNames)
                {
                    dirs.Add(Path.Combine(deployBase, GetPath(baseDir, dir)));
                }

                // Sort the string according to its length. Make sure the sub-directories are deleted first.
                dirs.Sort(new StringLengthComparer());

                try
                {
                    for (int index = dirs.Count - 1; index >= 0; --index)
                    {
                        string dir = (string)dirs[index];

                        // If the directory is empty and exists, delete it.
                        if (Directory.Exists(dir) && Directory.GetFileSystemEntries(dir).Length == 0)
                        {
                            Directory.Delete(dir);
                        }
                    }
                }
                catch (IOException e)
                {
                    throw new ServerException("Deleting the directory fails.", e);
                }
                catch (UnauthorizedAccessException e)
                {
                    throw new ServerException("No permission to access the directory.", e);
                }
            }
        }

        /// <summary>
        /// Gets the path of the copied file.
        /// </summary>
        /// <remarks>
        /// When the file is in the base directory, it is a relative path to the base directory. Otherwise, the full
        /// path is returned. This is exactly the same logic used in NAnt copy task.
        /// </remarks>
        /// <param name="baseDir">The base directory of the copied file.</param>
        /// <param name="file">The copied file whose path is retrieved.</param>
        /// <returns>The path of the copied file.</returns>
        private string GetPath(DirectoryInfo baseDir, string file)
        {
            // Get the full path of the file
            string path = Path.GetFullPath(file);

            if (path.StartsWith(baseDir.FullName))
            {
                path = path.Substring(baseDir.FullName.Length);
            }

            return (path[0] == Path.DirectorySeparatorChar) ? path.Substring(1) : path;
        }

        /// <summary>
        /// Gets the service controller and the maximum waiting time when changing the status of the service.
        /// </summary>
        /// <remarks>
        /// The service name is obtained using "service_name" in the option collection. When such option does not exist,
        /// "w3svc" is used. When the time out value does not exist, the default value 60000 is used. Empty option
        /// value is considered as if the option is missing.
        /// </remarks>
        /// <param name="controller">The controller of the service to be started/stopped.</param>
        /// <param name="timeOut">The maximum waiting time when changing the status of the service.</param>
        /// <exception cref="ServerException">The time out value exists, but it is not an non-negative integer.
        /// </exception>
        private void PrepareStartStop(out ServiceController controller, out int timeOut)
        {
            // Get the time out option
            Option option = (Options == null) ? null : Options[TIMEOUT];
            timeOut = ((option != null) && (option.Value.Trim().Length != 0)) ? Timeout : DEFAULT_TIMEOUT;

            // Get the controller option
            option = (Options == null) ? null : Options[SERVICE_NAME];
            controller = new ServiceController(((option != null) && (option.Value.Trim().Length != 0)) ?
                Options[SERVICE_NAME].Value : DEFAULT_SERVICE_NAME);

        }

        /// <summary>
        /// Initializes the data of the server.
        /// </summary>
        /// <remarks>
        /// This method is called only if the init flag is set to <c>true</c>. It is called after starting the server
        /// and before the tests. IIS server does not require data initialization.
        /// </remarks>
        public override void InitData()
        {
        }

        /// <summary>
        /// Cleans up the data of the server.
        /// </summary>
        /// <remarks>
        /// This method is called only if the init flag is set to <c>true</c>. It is called before stopping the server
        /// and after the tests. IIS server does not require data cleaning.
        /// </remarks>
        public override void CleanUpData()
        {
        }
    }
}

