/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using NUnit.Framework;
using NAnt.Core.Types;

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// Tests the functionality and error cases of <c>DatabaseServer</c> class. Since this class is an abstract class,
    /// a concrete implementation <c>MockDatabaseServer</c> is used in the tests.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class DatabaseServerTestCase
    {
        /// <summary>
        /// Represents the <c>DatabaseServer</c> instance used in tests.
        /// </summary>
        private DatabaseServer server;

        /// <summary>
        /// Sets up the test environment. The test instance is created.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            server = new MockDatabaseServer();
        }

        /// <summary>
        /// Cleans up the test environment. The test instance is disposed.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            server = null;
        }

        /// <summary>
        /// Tests accuracy of <c>DatabaseServer()</c>. Since the constructor is trivial, there should be no exception.
        /// </summary>
        [Test]
        public void TestDatabaseServerAccuracy()
        {
            Assert.IsNotNull(new MockDatabaseServer(), "A new instance of DatabaseServer should be created.");
        }

        /// <summary>
        /// Tests <c>DatabaseDriver</c> property getter when the option collection is not set. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestDatabaseDriverOptionCollectionNotSet()
        {
            string driver = server.DatabaseDriver;
        }

        /// <summary>
        /// Tests <c>DatabaseDriver</c> property getter when the option "db_driver" is missing. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestDatabaseDriverOptionMissing()
        {
            server.Options = new OptionCollection();

            string driver = server.DatabaseDriver;
        }

        /// <summary>
        /// Tests accuracy of <c>DatabaseDriver</c> property getter. The database driver should be "driver", as set in
        /// the option "db_driver".
        /// </summary>
        [Test]
        public void TestDatabaseDriverAccuracy()
        {
            Option option = new Option();

            option.OptionName = "db_driver";
            option.Value = "driver";
            server.Options = new OptionCollection();
            server.Options.Add(option);

            // Verify
            Assert.AreEqual("driver", server.DatabaseDriver,
                "The database driver should be defined in option 'db_driver'.");
        }

        /// <summary>
        /// Tests <c>DatabaseConnectionString</c> property getter when the option collection is not set. Expect
        /// ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestDatabaseConnectionStringOptionCollectionNotSet()
        {
            string connection = server.DatabaseConnectionString;
        }

        /// <summary>
        /// Tests <c>DatabaseConnectionString</c> property getter when the option "db_connection_str" is missing. Expect
        /// ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestDatabaseConnectionStringOptionMissing()
        {
            server.Options = new OptionCollection();

            string connection = server.DatabaseConnectionString;
        }

        /// <summary>
        /// Tests accuracy of <c>DatabaseConnectionString</c> property getter. The database connection string should be
        /// "connection", as set in the option "db_connection_str".
        /// </summary>
        [Test]
        public void TestDatabaseConnectionStringAccuracy()
        {
            Option option = new Option();

            option.OptionName = "db_connection_str";
            option.Value = "connection";
            server.Options = new OptionCollection();
            server.Options.Add(option);

            // Verify
            Assert.AreEqual("connection", server.DatabaseConnectionString,
                "The database connection string should be defined in option 'db_connection_str'.");
        }

        /// <summary>
        /// Tests <c>SqlInitScript</c> property getter when the option collection is not set. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestSqlInitScriptOptionCollectionNotSet()
        {
            string script = server.SqlInitScript;
        }

        /// <summary>
        /// Tests <c>SqlInitScript</c> property getter when the option "sql_init_script" is missing. Expect
        /// ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestSqlInitScriptOptionMissing()
        {
            server.Options = new OptionCollection();

            string script = server.SqlInitScript;
        }

        /// <summary>
        /// Tests accuracy of <c>SqlInitScript</c> property getter. The SQL init script should be "init script", as set
        /// in the option "sql_init_script".
        /// </summary>
        [Test]
        public void TestSqlInitScriptStringAccuracy()
        {
            Option option = new Option();

            option.OptionName = "sql_init_script";
            option.Value = "init script";
            server.Options = new OptionCollection();
            server.Options.Add(option);

            // Verify
            Assert.AreEqual("init script", server.SqlInitScript,
                "The SQL init script should be defined in option 'sql_init_script'.");
        }

        /// <summary>
        /// Tests <c>SqlCleanUpScript</c> property getter when the option collection is not set. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestSqlCleanUpScriptOptionCollectionNotSet()
        {
            string script = server.SqlCleanUpScript;
        }

        /// <summary>
        /// Tests <c>SqlCleanUpScript</c> property getter when the option "sql_cleanup_script" is missing. Expect
        /// ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestSqlCleanUpScriptOptionMissing()
        {
            server.Options = new OptionCollection();

            string script = server.SqlCleanUpScript;
        }

        /// <summary>
        /// Tests accuracy of <c>SqlCleanUpScript</c> property getter. The SQL init script should be "clean script", as
        /// set in the option "sql_cleanup_script".
        /// </summary>
        [Test]
        public void TestSqlCleanUpScriptStringAccuracy()
        {
            Option option = new Option();

            option.OptionName = "sql_cleanup_script";
            option.Value = "clean script";
            server.Options = new OptionCollection();
            server.Options.Add(option);

            // Verify
            Assert.AreEqual("clean script", server.SqlCleanUpScript,
                "The SQL clean up script should be defined in option 'sql_cleanup_script'.");
        }
    }
}

