/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using NUnit.Framework;
using NAnt.Core;
using NAnt.Core.Types;
using System.ServiceProcess;
using System.IO;

namespace TopCoder.Test.Framework.Servers.Web
{
    /// <summary>
    /// Tests the functionality and error cases of <c>IISServer</c> class. Since this class is extended from
    /// <c>ApplicationServer</c> class, all methods in <c>ApplicationServer</c> class are not tested here. Some errors
    /// can hardly be produced by a program, such as <c>Win32Exception</c>. The time out cannot be tested as well, since
    /// the computer might be fast enough to start/stop the service despite any time out value (even 0 or 1
    /// millisecond).
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class IISServerTestCase
    {
        /// <summary>
        /// Represents the <c>IISServer</c> instance used in tests.
        /// </summary>
        private IISServer server;

        /// <summary>
        /// Represents the NAnt project used in tests.
        /// </summary>
        private Project project;

        /// <summary>
        /// Represents the service controller for IIS server.
        /// </summary>
        private ServiceController controller;

        /// <summary>
        /// Cleans up the test web site. The web site directory is deleted. The IIS server is stopped.
        /// </summary>
        private void CleanWebSite()
        {
            // Stop the IIS server
            controller.Refresh();

            switch (controller.Status)
            {
                case ServiceControllerStatus.StartPending:
                case ServiceControllerStatus.ContinuePending:
                    controller.WaitForStatus(ServiceControllerStatus.Running);

                    break;

                case ServiceControllerStatus.StopPending:
                    controller.WaitForStatus(ServiceControllerStatus.Stopped);

                    break;

                case ServiceControllerStatus.PausePending:
                    controller.WaitForStatus(ServiceControllerStatus.Paused);

                    break;
            }

            if (controller.CanStop)
            {
                controller.Stop();
                controller.WaitForStatus(ServiceControllerStatus.Stopped);
            }

            // Delete all files in the web site directory.
            foreach (FileInfo info in new DirectoryInfo("../../test_files/website").GetFiles())
            {
                info.Delete();
            }

            // Recursively delete all directories in the web site directory.
            foreach (DirectoryInfo dir in new DirectoryInfo("../../test_files/website").GetDirectories())
            {
                dir.Delete(true);
            }
        }

        /// <summary>
        /// Sets up the test environment. The test instance is created. The IIS server is stopped.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            project = new Project("../../test_files/default.build", Level.Error);
            server = new IISServer();
            server.Project = project;
            controller = new ServiceController("w3svc");

            CleanWebSite();
        }

        /// <summary>
        /// Cleans up the test environment. The test instance is disposed. The IIS server is stopped.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            CleanWebSite();

            controller.Dispose();
            server = null;
            project = null;
            controller = null;
        }

        /// <summary>
        /// Copies all files in the source directory to the target directory.
        /// </summary>
        /// <param name="source">The source directory whose files are copied.</param>
        /// <param name="target">The target directory where the files are copied.</param>
        private void CopyFiles(DirectoryInfo source, DirectoryInfo target)
        {
            foreach (FileInfo file in source.GetFiles())
            {
                file.CopyTo(Path.Combine(target.FullName, Path.GetFileName(file.FullName)));
            }

            // Recursively copy the files
            foreach (DirectoryInfo dir in source.GetDirectories())
            {
                DirectoryInfo targetDir = new DirectoryInfo(Path.Combine(target.FullName, dir.Name));

                targetDir.Create();
                CopyFiles(dir, targetDir);
            }
        }

        /// <summary>
        /// Tests accuracy of <c>IISServer()</c>. Since the constructor is trivial, there should be no exception.
        /// </summary>
        [Test]
        public void TestIISServerAccuracy()
        {
            Assert.IsNotNull(new IISServer(), "A new instance of IISServer should be created.");
        }

        /// <summary>
        /// Tests <c>StartUp()</c> when the deployment directory is not set. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestStartUpDeploymentNotSet()
        {
            // Set server properties
            server.Options = new OptionCollection();

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Start the server
            server.StartUp();
        }

        /// <summary>
        /// Tests <c>StartUp()</c> when deploying the web site fails. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestStartUpCopyFail()
        {
            // Set server properties
            Option option = new Option();

            server.Options = new OptionCollection();
            option.OptionName = "deployment_dir";
            option.Value = "default.build";
            server.Options.Add(option);

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Start the server
            server.StartUp();
        }

        /// <summary>
        /// Tests accuracy of <c>StartUp()</c> when the source files are not set. The IIS server should be started. No
        /// files should be copied.
        /// </summary>
        [Test]
        public void TestStartUpSourceFilesNoCopyAccuracy()
        {
            // Set no copy
            server.InitServer = false;

            // Start the server
            server.StartUp();

            // Verify the server status
            controller.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Running, controller.Status, "The IIS service should be running.");
        }

        /// <summary>
        /// Tests <c>StartUp()</c> when the IIS service cannot be found. Expect ServerException. This also tests the
        /// ability to specify service name of the IIS service.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestStartUpServiceNotFound()
        {
            // Set server properties
            Option option = new Option();

            server.Options = new OptionCollection();
            option.OptionName = "deployment_dir";
            option.Value = "website";
            server.Options.Add(option);

            option = new Option();
            option.OptionName = "service_name";
            option.Value = "unknown";
            server.Options.Add(option);

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Start the server
            server.StartUp();
        }

        /// <summary>
        /// Tests <c>StartUp()</c> when the IIS service is already started. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestStartUpServiceStarted()
        {
            // Start the service
            controller.Start();
            controller.WaitForStatus(ServiceControllerStatus.Running);

            // Set server properties
            Option option = new Option();

            server.Options = new OptionCollection();
            option.OptionName = "deployment_dir";
            option.Value = "website";
            server.Options.Add(option);

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Start the server
            server.StartUp();
        }

        /// <summary>
        /// Tests <c>StartUp()</c> when the time out value is invalid. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestStartUpTimeoutInvalid()
        {
            // Set server properties
            Option option = new Option();

            server.Options = new OptionCollection();
            option.OptionName = "deployment_dir";
            option.Value = "website";
            server.Options.Add(option);

            // Set the time out to 0 millisecond
            option = new Option();
            option.OptionName = "timeout";
            option.Value = "abc";
            server.Options.Add(option);

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Start the server
            server.StartUp();
        }

        /// <summary>
        /// Tests accuracy of <c>StartUp()</c> when the IIS service name and the time out value are missing in the
        /// options, and the IIS service is stopped. The IIS service should enter the running status. The two files
        /// and one directory should be copied to the deployment directory. The service name "w3svc" should be used.
        /// </summary>
        [Test]
        public void TestStartUpStoppedAccuracy()
        {
            // Set server properties
            Option option = new Option();

            server.Options = new OptionCollection();
            option.OptionName = "deployment_dir";
            option.Value = "website";
            server.Options.Add(option);

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Start the server
            server.StartUp();

            // Verify the server status
            controller.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Running, controller.Status, "The IIS service should be running.");

            // Verify the copied files
            DirectoryInfo info = new DirectoryInfo("../../test_files/website");

            Assert.AreEqual(2, info.GetFiles().Length, "Two files should be copied to the deployment directory.");
            Assert.AreEqual(1, info.GetDirectories().Length,
                "One directory should be copied to the deployment directory.");
        }

        /// <summary>
        /// Tests accuracy of <c>StartUp()</c> when the IIS service name and the time out value are empty string in the
        /// options, and the IIS service is paused. The IIS service should enter the running status. The two files and
        /// one directory should be copied to the deployment directory. The service name "w3svc" should be used.
        /// </summary>
        [Test]
        public void TestStartUpPausedAccuracy()
        {
            // Start and pause the service
            controller.Start();
            controller.WaitForStatus(ServiceControllerStatus.Running);
            controller.Pause();
            controller.WaitForStatus(ServiceControllerStatus.Paused);

            // Set server properties
            Option option = new Option();

            server.Options = new OptionCollection();
            option.OptionName = "deployment_dir";
            option.Value = "website";
            server.Options.Add(option);

            // Set time out and service name
            option = new Option();
            option.OptionName = "service_name";
            option.Value = "   ";
            server.Options.Add(option);
            option = new Option();
            option.OptionName = "timeout";
            option.Value = "   ";
            server.Options.Add(option);

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Start the server
            server.StartUp();

            // Verify the server status
            controller.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Running, controller.Status, "The IIS service should be running.");

            // Verify the copied files
            DirectoryInfo info = new DirectoryInfo("../../test_files/website");

            Assert.AreEqual(2, info.GetFiles().Length, "Two files should be copied to the deployment directory.");
            Assert.AreEqual(1, info.GetDirectories().Length,
                "One directory should be copied to the deployment directory.");
        }

        /// <summary>
        /// Tests accuracy of <c>StartUp()</c> when the IIS service name and the time out value are specified in the
        /// options, and the IIS service is paused. The IIS service should enter the running status. The two files and
        /// one directory should be copied to the deployment directory. The service name and the time out value in the
        /// options should be used.
        /// </summary>
        [Test]
        public void TestStartUpOptionSpecifiedAccuracy()
        {
            // Start and pause the service
            controller.Start();
            controller.WaitForStatus(ServiceControllerStatus.Running);
            controller.Pause();
            controller.WaitForStatus(ServiceControllerStatus.Paused);

            // Set server properties
            Option option = new Option();

            server.Options = new OptionCollection();
            option.OptionName = "deployment_dir";
            option.Value = "website";
            server.Options.Add(option);

            // Set time out and service name
            option = new Option();
            option.OptionName = "service_name";
            option.Value = "w3svc";
            server.Options.Add(option);
            option = new Option();
            option.OptionName = "timeout";
            option.Value = "10000";
            server.Options.Add(option);

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Start the server
            server.StartUp();

            // Verify the server status
            controller.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Running, controller.Status, "The IIS service should be running.");

            // Verify the copied files
            DirectoryInfo info = new DirectoryInfo("../../test_files/website");

            Assert.AreEqual(2, info.GetFiles().Length, "Two files should be copied to the deployment directory.");
            Assert.AreEqual(1, info.GetDirectories().Length,
                "One directory should be copied to the deployment directory.");
        }

        /// <summary>
        /// Tests <c>ShutDown()</c> when the deployment directory is not set. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestShutDownDeploymentNotSet()
        {
            // Start the service
            controller.Start();
            controller.WaitForStatus(ServiceControllerStatus.Running);

            // Set server properties
            server.Options = new OptionCollection();

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Stop the server
            server.ShutDown();
        }

        /// <summary>
        /// Tests <c>ShutDown()</c> when deleting the web site fails. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestShutDownDeleteFail()
        {
            // Start the service
            controller.Start();
            controller.WaitForStatus(ServiceControllerStatus.Running);

            // Copy the files
            CopyFiles(new DirectoryInfo("../../test_files/files"), new DirectoryInfo("../../test_files/website"));

            // Set server properties
            Option option = new Option();

            server.Options = new OptionCollection();
            option.OptionName = "deployment_dir";
            option.Value = "website";
            server.Options.Add(option);

            // Set files to be deleted
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Open one file
            Stream stream = File.OpenWrite("../../test_files/website/test.aspx");

            // Stop the server
            try
            {
                server.ShutDown();
            }
            finally
            {
                stream.Close();
            }
        }

        /// <summary>
        /// Tests accuracy of <c>ShutDown()</c> when the source files are not copied. The IIS server should be stopped.
        /// No files should be deleted.
        /// </summary>
        [Test]
        public void TestShutDownSourceNoCopyAccuracy()
        {
            // Start the service
            controller.Start();
            controller.WaitForStatus(ServiceControllerStatus.Running);

            // Set no init server
            server.InitServer = false;

            // Stop the server
            server.ShutDown();

            // Verify the server status
            controller.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Stopped, controller.Status, "The IIS service should be stopped.");
        }

        /// <summary>
        /// Tests <c>ShutDown()</c> when the IIS service cannot be found. Expect ServerException. This also tests the
        /// ability to specify service name of the IIS service.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestShutDownServiceNotFound()
        {
            // Set server properties
            Option option = new Option();

            server.Options = new OptionCollection();
            option.OptionName = "deployment_dir";
            option.Value = "website";
            server.Options.Add(option);

            option = new Option();
            option.OptionName = "service_name";
            option.Value = "unknown";
            server.Options.Add(option);

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Copy the files
            CopyFiles(new DirectoryInfo("../../test_files/files"), new DirectoryInfo("../../test_files/website"));

            // Stop the server
            server.ShutDown();
        }

        /// <summary>
        /// Tests <c>ShutDown()</c> when the IIS service is already stopped. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestShutDownServiceStopped()
        {
            // Set server properties
            Option option = new Option();

            server.Options = new OptionCollection();
            option.OptionName = "deployment_dir";
            option.Value = "website";
            server.Options.Add(option);

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Copy the files
            CopyFiles(new DirectoryInfo("../../test_files/files"), new DirectoryInfo("../../test_files/website"));

            // Stop the server
            server.ShutDown();
        }

        /// <summary>
        /// Tests <c>ShutDown()</c> when the time out value is invalid. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestShutDownTimeoutInvalid()
        {
            // Start the service
            controller.Start();
            controller.WaitForStatus(ServiceControllerStatus.Running);

            // Set server properties
            Option option = new Option();

            server.Options = new OptionCollection();
            option.OptionName = "deployment_dir";
            option.Value = "website";
            server.Options.Add(option);

            // Set the time out to 0 millisecond
            option = new Option();
            option.OptionName = "timeout";
            option.Value = "abc";
            server.Options.Add(option);

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Copy the files
            CopyFiles(new DirectoryInfo("../../test_files/files"), new DirectoryInfo("../../test_files/website"));

            // Stop the server
            server.ShutDown();
        }

        /// <summary>
        /// Tests accuracy of <c>ShutDown()</c> when the IIS service name and the time out value are missing in the
        /// options, and the IIS service is running. The IIS service should be stopped. The copied files should be
        /// deleted from the deployment directory. The service name "w3svc" should be used. There should be one file
        /// left in the deployment directory. The directories should not be deleted, since there is an extra file.
        /// </summary>
        [Test]
        public void TestShutDownOptionMissingAccuracy()
        {
            // Start the service
            controller.Start();
            controller.WaitForStatus(ServiceControllerStatus.Running);

            // Copy the files
            CopyFiles(new DirectoryInfo("../../test_files/files"), new DirectoryInfo("../../test_files/website"));
            File.Copy("../../test_files/files/Press.txt", "../../test_files/website/Press1.txt");
            File.Copy("../../test_files/files/Press.txt", "../../test_files/website/Dir/Dir1/Press5.txt");

            // Set server properties
            Option option = new Option();

            server.Options = new OptionCollection();
            option.OptionName = "deployment_dir";
            option.Value = "website";
            server.Options.Add(option);

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Stop the server
            server.ShutDown();

            // Verify the server status
            controller.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Stopped, controller.Status, "The IIS service should be stopped.");

            // Verify the copied files
            DirectoryInfo info = new DirectoryInfo("../../test_files/website");

            Assert.AreEqual(1, info.GetFiles().Length, "Files should be deleted from the deployment directory.");
            Assert.AreEqual(1, info.GetDirectories().Length, "The sub-directory should not be deleted.");
            Assert.AreEqual(1, info.GetDirectories()[0].GetDirectories().Length,
                "The directory in the sub-directory should not be deleted.");
            Assert.AreEqual(1, info.GetDirectories()[0].GetDirectories()[0].GetFiles().Length,
                "The file in sub-directory should not be deleted.");
        }

        /// <summary>
        /// Tests accuracy of <c>ShutDown()</c> when the IIS service name and the time out value are empty string in the
        /// options, and the IIS service is running. The IIS service should be stopped. The copied files and directory
        /// should be deleted from the deployment directory. The service name "w3svc" should be used. There should be
        /// one file left in the deployment directory. The 'Dir' directory should not be deleted, since there is an
        /// extra file.
        /// </summary>
        [Test]
        public void TestShutDownOptionEmptyAccuracy()
        {
            // Start the service
            controller.Start();
            controller.WaitForStatus(ServiceControllerStatus.Running);

            // Copy the files
            CopyFiles(new DirectoryInfo("../../test_files/files"), new DirectoryInfo("../../test_files/website"));
            File.Copy("../../test_files/files/Press.txt", "../../test_files/website/Press1.txt");
            File.Copy("../../test_files/files/Press.txt", "../../test_files/website/Dir/Press.txt");

            // Set server properties
            Option option = new Option();

            server.Options = new OptionCollection();
            option.OptionName = "deployment_dir";
            option.Value = "website";
            server.Options.Add(option);

            // Set time out and service name
            option = new Option();
            option.OptionName = "service_name";
            option.Value = "   ";
            server.Options.Add(option);
            option = new Option();
            option.OptionName = "timeout";
            option.Value = "   ";
            server.Options.Add(option);

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Stop the server
            server.ShutDown();

            // Verify the server status
            controller.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Stopped, controller.Status, "The IIS service should be stopped.");

            // Verify the copied files
            DirectoryInfo info = new DirectoryInfo("../../test_files/website");

            Assert.AreEqual(1, info.GetFiles().Length, "Files should be deleted from the deployment directory.");
            Assert.AreEqual(1, info.GetDirectories().Length, "The sub-directory should not be deleted.");
            Assert.AreEqual(1, info.GetDirectories()[0].GetFiles().Length,
                "The file in sub-directory should not be deleted.");
            Assert.AreEqual(0, info.GetDirectories()[0].GetDirectories().Length,
                "The directory in the sub-directory should be deleted.");
        }

        /// <summary>
        /// Tests accuracy of <c>ShutDown()</c> when the IIS service name and the time out value are specified in the
        /// options, and the IIS service is running. The IIS service should be stopped. The two files and one directory
        /// should be deleted from the deployment directory. The service name and the time out value in the options
        /// should be used. There should be one file left in the deployment directory.
        /// </summary>
        [Test]
        public void TestShutDownOptionSpecifiedAccuracy()
        {
            // Start and pause the service
            controller.Start();
            controller.WaitForStatus(ServiceControllerStatus.Running);

            // Copy the files
            CopyFiles(new DirectoryInfo("../../test_files/files"), new DirectoryInfo("../../test_files/website"));
            File.Copy("../../test_files/files/Press.txt", "../../test_files/website/Press1.txt");

            // Set server properties
            Option option = new Option();

            server.Options = new OptionCollection();
            option.OptionName = "deployment_dir";
            option.Value = "website";
            server.Options.Add(option);

            // Set time out and service name
            option = new Option();
            option.OptionName = "service_name";
            option.Value = "w3svc";
            server.Options.Add(option);
            option = new Option();
            option.OptionName = "timeout";
            option.Value = "10000";
            server.Options.Add(option);

            // Set files to be copied
            server.SourceFiles = new FileSet();
            server.SourceFiles.BaseDirectory = "../../test_files/files";
            server.SourceFiles.Includes.Add("**/*");

            // Stop the server
            server.ShutDown();

            // Verify the server status
            controller.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Stopped, controller.Status, "The IIS service should be stopped.");

            // Verify the copied files
            DirectoryInfo info = new DirectoryInfo("../../test_files/website");

            Assert.AreEqual(1, info.GetFiles().Length, "Files should be deleted from the deployment directory.");
        }

        /// <summary>
        /// Tests accuracy of <c>InitData()</c>. This is a trivial method. It should not throw any exception.
        /// </summary>
        [Test]
        public void TestInitDataAccuracy()
        {
            server.InitData();
        }

        /// <summary>
        /// Tests accuracy of <c>CleanUpData()</c>. This is a trivial method. It should not throw any exception.
        /// </summary>
        [Test]
        public void TestCleanUpDataAccuracy()
        {
            server.CleanUpData();
        }
    }
}

